/*!
 * VERSION: 3.0.0
 * DATE: 2019-09-20
 * http://greensock.com
 *
 * Copyright (c) 2008-2019, GreenSock. All rights reserved.
 * This work is subject to the terms at http://greensock.com/standard-license or for
 * Club GreenSock members, the agreement that was issued with that membership.
 * @author: Jack Doyle, jack@greensock.com
*/
/* eslint-disable */

let _doc,
	_setDoc = element => {
		let doc = element.ownerDocument || element;
		while (doc.parentNode && (doc = doc.parentNode)) {	}
		return (_doc = doc);
	},
	_svgTemps = [], //we create 3 elements for SVG, and 3 for other DOM elements and cache them for performance reasons. They get nested in _divContainer and _svgContainer so that just one element is added to the DOM on each successive attempt. Again, performance is key.
	_divTemps = [],
	_divContainer,
	_svgContainer,
	_identityMatrix = {a:1, b:0, c:0, d:1, e:0, f:0},
	_svgOwner = element => element.ownerSVGElement || (element.tagName.toLowerCase() === "svg" ? element : null),
	_createSibling = (element, i) => {
		if (element.parentNode && (_doc || _setDoc(element))) {
			let svg = _svgOwner(element),
				ns = svg ? (svg.getAttribute("xmlns") || "http://www.w3.org/2000/svg") : "http://www.w3.org/1999/xhtml",
				type = svg ? (i ? "rect" : "g") : "div",
				x = i !== 2 ? 0 : 100,
				y = i === 3 ? 100 : 0,
				css = "position:absolute;display:block;",
				e = _doc.createElementNS ? _doc.createElementNS(ns.replace(/^https/, "http"), type) : _doc.createElement(type);
			if (i) {
				if (!svg) {
					if (!_divContainer) {
						_divContainer = _createSibling(element);
						_divContainer.style.cssText = css;
					}
					e.style.cssText = css + "width:1px;height:1px;top:" + y + "px;left:" + x + "px";
					_divContainer.appendChild(e);

				} else {
					if (!_svgContainer) {
						_svgContainer = _createSibling(element);
					}
					e.setAttribute("width", 1);
					e.setAttribute("height", 1);
					e.setAttribute("transform", "translate(" + x + "," + y + ")");
					_svgContainer.appendChild(e);
				}
			}
			return e;
		}
		throw "Need document and parent.";
	},
	_placeSiblings = element => {
		let svg = _svgOwner(element),
			isRootSVG = element === svg,
			siblings = svg ? _svgTemps : _divTemps,
			container, m, b;
		if (!siblings.length) {
			siblings.push(_createSibling(element, 1), _createSibling(element, 2), _createSibling(element, 3));
		}
		container = svg ? _svgContainer : _divContainer;
		if (svg) {
			b = isRootSVG ? {x:0, y:0} : element.getBBox();
			m = element.transform.baseVal;
			m = m.length ? m.consolidate().matrix : _identityMatrix;
			container.setAttribute("transform", "matrix(" + m.a + "," + m.b + "," + m.c + "," + m.d + "," + (m.e + b.x) + "," + (m.f + b.y) + ")");
			(isRootSVG ? svg : element.parentNode).appendChild(container);
		} else {
			container.style.top = element.offsetTop + "px";
			container.style.left = element.offsetLeft + "px";
			container.style.transform = window.getComputedStyle(element).transform;
			element.parentNode.appendChild(container);
		}
		return container;
	};

//feed in an element and it'll return a 2D matrix (optionally inverted) so that you can translate between coordinate spaces.
// Inverting lets you translate a global point into a local coordinate space. No inverting lets you go the other way.
// We needed this to work around various browser bugs, like Firefox doesn't accurately report getScreenCTM() when there
// are transforms applied to ancestor elements.
// The matrix math to convert any x/y coordinate is:
//     tx = m[0] * x + m[2] * y + m[4]
//     ty = m[1] * x + m[3] * y + m[5]
export function getGlobalMatrix(element, invert) {
	let svg = _svgOwner(element),
		temps = svg ? _svgTemps : _divTemps,
		container = _placeSiblings(element),
		b1 = temps[0].getBoundingClientRect(),
		b2 = temps[1].getBoundingClientRect(),
		b3 = temps[2].getBoundingClientRect(),
		a = (b2.left - b1.left) / 100,
		b = (b2.top - b1.top) / 100,
		c = (b3.left - b1.left) / 100,
		d = (b3.top - b1.top) / 100,
		tx = b1.left,
		ty = b1.top,
		determinant;
	container.parentNode.removeChild(container);
	if (invert) {
		determinant = (a * d - b * c);
		return [
			d / determinant,
			-b / determinant,
			-c / determinant,
			a / determinant,
			(c * ty - d * tx) / determinant,
			-(a * ty - b * tx) / determinant
		];
	}
	return [a, b, c, d, tx, ty];
}